from panda3d.core import *
#load_prc_file_data("", "gl-version 3 3")

from array import array
from random import random

NUM_EMBERS = 512

# Create the SSBO and populate it with random data.
floats = []
for i in range(NUM_EMBERS):
    speed = (0.2 + random()) * 0.7
    size = 0.5 + random() * 0.5
    # Make bigger ones rarer
    size = max(size ** 4, 0.1)
    floats += [random(), random(), random(), speed, 0, 0, 0, size]

initial_data = array('f', floats)
buffer = ShaderBuffer('embers', initial_data.tobytes(), GeomEnums.UH_static)

# Create a dummy vertex data object.
format = GeomVertexFormat.get_empty()
vdata = GeomVertexData('embers', format, GeomEnums.UH_static)

# This represents a draw call, indicating how many vertices we want to draw.
tris = GeomTriangles(GeomEnums.UH_static)
tris.add_next_vertices(NUM_EMBERS * 3)

# We need to set a bounding volume so that Panda doesn't try to cull it.
geom = Geom(vdata)
geom.add_primitive(tris)
geom.set_bounds(BoundingBox((0, 0, 0), (1, 1, 1)))

node = GeomNode("node")
node.add_geom(geom)

from direct.showbase.ShowBase import ShowBase

base = ShowBase()
base.set_background_color(0, 0, 0)

embertex = loader.load_texture("ember.png")
embertex.wrap_u = SamplerState.WM_clamp
embertex.wrap_v = SamplerState.WM_clamp

# Put our node in the scene graph and apply the SSBO.
shader = Shader.load(Shader.SL_GLSL, "embers.vert", "embers.frag")
path = base.render.attach_new_node(node)
path.set_shader(shader)
path.set_shader_input("EmberBuffer", buffer)
path.set_texture(embertex)
path.set_two_sided(True)
path.set_attrib(ColorBlendAttrib.make(ColorBlendAttrib.M_add, ColorBlendAttrib.O_incoming_alpha, ColorBlendAttrib.O_one))
path.set_depth_write(False)
path.node().set_bounds_type(BoundingVolume.BT_box)
#path.show_bounds()

# Add the ComputeNode to do the simulation.
node = ComputeNode("simulate")
node.add_dispatch(NUM_EMBERS // 16, 1, 1)
path = base.render.attach_new_node(node)
path.set_shader(Shader.load_compute(Shader.SL_GLSL, "simulate.comp"))
path.set_shader_input("windmap", loader.load_texture("wind.png"))
path.set_shader_input("EmberBuffer", buffer)

base.cam.set_x(0.5)
base.cam.set_y(-2)
base.cam.set_z(0.5)

base.run()
